/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.flink.procedure;

import org.apache.paimon.catalog.Catalog;
import org.apache.paimon.catalog.Identifier;
import org.apache.paimon.table.Table;

import org.apache.flink.table.annotation.ArgumentHint;
import org.apache.flink.table.annotation.DataTypeHint;
import org.apache.flink.table.annotation.ProcedureHint;
import org.apache.flink.table.procedure.ProcedureContext;

/**
 * Create tag procedure. Usage:
 *
 * <pre><code>
 *  CALL sys.rename_tag('tableId', 'tagName', 'targetTagName')
 * </code></pre>
 */
public class RenameTagProcedure extends ProcedureBase {
    private static final String IDENTIFIER = "rename_tag";

    @ProcedureHint(
            argument = {
                @ArgumentHint(name = "table", type = @DataTypeHint("STRING")),
                @ArgumentHint(name = "tagName", type = @DataTypeHint("STRING")),
                @ArgumentHint(name = "targetTagName", type = @DataTypeHint("STRING"))
            })
    public String[] call(
            ProcedureContext procedureContext, String tableId, String tagName, String targetTagName)
            throws Catalog.TableNotExistException {
        Table table = catalog.getTable(Identifier.fromString(tableId));
        table.renameTag(tagName, targetTagName);
        String ret = String.format("Rename [%s] to [%s] successfully.", tagName, targetTagName);
        return new String[] {ret};
    }

    @Override
    public String identifier() {
        return IDENTIFIER;
    }
}
