/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.flink.source;

import org.apache.flink.api.connector.source.SourceSplit;

import java.util.UUID;

/** A {@link SourceSplit} that provides basic information through splitId. */
public class SimpleSourceSplit implements SourceSplit {
    private final String splitId;
    private final String value;

    public SimpleSourceSplit() {
        this("");
    }

    public SimpleSourceSplit(String value) {
        this(UUID.randomUUID().toString(), value);
    }

    public SimpleSourceSplit(String splitId, String value) {
        this.splitId = splitId;
        this.value = value;
    }

    @Override
    public String splitId() {
        return splitId;
    }

    public String value() {
        return value;
    }
}
